package cz.jzitnik;

import cz.jzitnik.game.Game;
import cz.jzitnik.game.MouseHandler;
import cz.jzitnik.game.SpriteLoader;
import cz.jzitnik.tui.ScreenRenderer;
import org.jline.terminal.MouseEvent;
import org.jline.terminal.Terminal;
import org.jline.terminal.TerminalBuilder;

import java.io.IOException;

public class Main {
    public static void main(String[] args) {
        try {
            Terminal terminal = TerminalBuilder.terminal();
            terminal.enterRawMode();
            terminal.trackMouse(Terminal.MouseTracking.Any);

            if (!terminal.hasMouseSupport()) {
                System.out.println("Error: This terminal does not support mouse.");
                System.exit(1);
            }

            var spriteList = SpriteLoader.load();
            var screenRenderer = new ScreenRenderer(spriteList, terminal);
            var game = new Game();

            MouseHandler mouseHandler = new MouseHandler(game, terminal, screenRenderer);

            final boolean[] isRunning = {true};

            Thread inputThread = new Thread(() -> {
                try {
                    while (isRunning[0]) {
                        int key = terminal.reader().read();

                        // Check for mouse event sequence
                        if (key == 27) { // ESC
                            key = terminal.reader().read();
                            if (key == '[') {
                                key = terminal.reader().read();
                                if (key == 'M') {
                                    MouseEvent mouseEvent = terminal.readMouseEvent();
                                    mouseHandler.handle(mouseEvent);
                                }
                            }
                        }

                        switch (key) {
                            case 'a':
                                game.movePlayerLeft(screenRenderer);
                                screenRenderer.render(game);
                                break;
                            case 'd':
                                game.movePlayerRight(screenRenderer);
                                screenRenderer.render(game);
                                break;
                            case ' ':
                                game.movePlayerUp(screenRenderer);
                                screenRenderer.render(game);
                                break;
                            case 'q':
                                System.out.println("Exiting game...");
                                isRunning[0] = false;
                                break;
                            default:
                                break;
                        }
                    }
                } catch (IOException e) {
                    e.printStackTrace();
                }
            });

            inputThread.start();

            // Game loop (rendering the game)
            while (isRunning[0]) {
                screenRenderer.render(game);
                try {
                    Thread.sleep(1000);
                } catch (InterruptedException e) {
                    e.printStackTrace();
                }
            }

            // Disable mouse tracking on exit
            terminal.trackMouse(Terminal.MouseTracking.Off);

            terminal.close();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }
}
