package cz.jzitnik.game;

import cz.jzitnik.game.items.InventoryItem;
import cz.jzitnik.game.items.Item;
import cz.jzitnik.tui.SpriteCombiner;
import cz.jzitnik.game.sprites.Number;
import cz.jzitnik.tui.SpriteList;
import lombok.Getter;
import org.jline.terminal.Terminal;

import java.util.ArrayList;
import java.util.List;
import java.util.Optional;

@Getter
public class Inventory {
    private static final int INVENTORY_SIZE_PX = 470;

    private List<InventoryItem> items = new ArrayList<>();
    private int itemInhHandIndex = 0;

    public Optional<Item> getItemInHand() {

        return Optional.empty();
    }

    public void addItem(Item item) {
        if (!item.isStackable()) {
            items.add(new InventoryItem(item));
            return;
        }

        var it = items.stream().filter(i -> i.getItem().equals(item)).toList();
        if (!it.isEmpty()) {
            it.get(0).setAmount(it.get(0).getAmount() + 1);

            return;
        }

        items.add(new InventoryItem(item));
    }

    private Number.NumberState getNumberState(int digit) {
        return switch (digit) {
            case 0 -> Number.NumberState.ZERO;
            case 1 -> Number.NumberState.ONE;
            case 2 -> Number.NumberState.TWO;
            case 3 -> Number.NumberState.THREE;
            case 4 -> Number.NumberState.FOUR;
            case 5 -> Number.NumberState.FIVE;
            case 6 -> Number.NumberState.SIX;
            case 7 -> Number.NumberState.SEVEN;
            case 8 -> Number.NumberState.EIGHT;
            case 9 -> Number.NumberState.NINE;
            default -> throw new IllegalArgumentException("Unexpected number: " + digit);
        };
    }

    private String getNumberSprite(int number) {
        Number nm = new Number();
        if (number <= 9) {
            Number.NumberState numberState = getNumberState(number);

            return Number.fixForHotbar(nm.getSprite(numberState), 1);
        }

        String numStr = Integer.toString(number); // Convert to string
        char firstChar = numStr.charAt(0); // Get first digit
        char secondChar = numStr.charAt(1); // Get second digit

        int firstDigit = Character.getNumericValue(firstChar);
        int secondDigit = Character.getNumericValue(secondChar);

        Number.NumberState numberState1 = getNumberState(firstDigit);
        Number.NumberState numberState2 = getNumberState(secondDigit);

        return SpriteCombiner.combineTwoSprites( Number.fixForHotbar(nm.getSprite(numberState1), 2), Number.fixForHotbar(nm.getSprite(numberState2), 1));
    }

    public void renderHotbar(StringBuilder buffer, SpriteList spriteList, Terminal terminal) {
        int termWidth = terminal.getWidth();
        int startLeft = (termWidth / 2) - (INVENTORY_SIZE_PX / 2);

        List<String> sprites = items.subList(0, Math.min(9, items.size())).stream().map(
                items -> SpriteCombiner.combineTwoSprites(
                        spriteList.getSprite(items.getItem().getSprite()).getSprite(),
                        getNumberSprite(items.getAmount())
                )
        ).toList();

        for (int i = 0; i < 26; i++) {
            // Empty left space
            buffer.append("\033[0m ".repeat(Math.max(0, startLeft)));

            if (i == 0 || i == 25) {
                buffer.append("\033[38;5;231;48;5;231m▓".repeat(INVENTORY_SIZE_PX - 2));
            } else {
                for (int j = 0; j < 9; j++) {
                    buffer.append("\033[38;5;231;48;5;231m▓".repeat(2));
                    if (sprites.size() <= j) {
                        buffer.append("\033[0m ".repeat(50));
                        continue;
                    }

                    String x = sprites.get(j).split("\n")[i];
                    buffer.append(x);
                }
            }
            buffer.append("\033[38;5;231;48;5;231m▓".repeat(2));
            buffer.append("\033[0m\n");
        }
    }
}
